
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include <iostream>

#include "Character.hh"
#include "Log.hh"

	
Character::Character(const std::string& name) :
	tilemap_(name),
	animation_(name)
{
	current.mass = 1.0;
	current.inverseMass = 1.0 / current.mass;

	// gravity
	current.force = Mf::Vector2(0.0, -120.0);

	// starting position
	current.position = Mf::Vector2(64.0, 64.0);
	current.momentum = Mf::Vector2(0.0, 0.0);
	current.recalculate();

	previous = current;

	updateContainers();
}


void Character::update(Mf::Scalar t, Mf::Scalar dt)
{
	previous = current;

	Mf::Vector2 x = current.position - Mf::Vector2(500.0, 200.0);
	Mf::Scalar mag = x.length();
	Mf::Scalar d = 50.0;

	current.force = Mf::Vector2(0.0, -2000.0);
	//current.force += -15.0 * x - 1.5 * current.velocity;
	current.force += -20.0 * (mag - d) * (x / mag) - 2.0 * current.velocity;
	current.force += userForce;
	current.recalculate();
	std::cout << "force: " << current.momentum << std::endl;

	Mf::integrate<State,Derivative>(current, t, dt);

	animation_.update(t, dt);

	updateContainers();
}

void Character::updateContainers()
{
	aabb_.init(Mf::Vector3(current.position[0]-16.0, current.position[1]-16.0, z),
			Mf::Vector3(current.position[0]+16.0, current.position[1]+16.0, z));
	sphere_.point = Mf::Vector3(current.position[0], current.position[1], z);
	sphere_.radius = (aabb_.min - sphere_.point).length();
}

void Character::handleEvent(const Mf::Event& event)
{
	// really just for heroine...
	
	Mf::Scalar force = 4000.0;
	
	Mf::Vector2 left = Mf::Vector2(-force, 0.0);
	Mf::Vector2 right = Mf::Vector2(force, 0.0);
	Mf::Vector2 down = Mf::Vector2(0.0, -force);
	Mf::Vector2 up = Mf::Vector2(0.0, force);

	switch (event.type)
	{
		case SDL_KEYDOWN:
			if (event.key.keysym.sym == SDLK_a)
			{
				userForce += left;
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				userForce += right;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				userForce += down;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				userForce += up;
			}
			break;

		case SDL_KEYUP:
			if (event.key.keysym.sym == SDLK_a)
			{
				userForce -= left;
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				userForce -= right;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				userForce -= down;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				userForce -= up;
			}
			break;
	}

	//Mf::logInfo("current force [%f %f]", current.force[0], current.force[1]);
	//std::cerr << "current force: " << current.force << std::endl;
}


void Character::draw(Mf::Scalar alpha) const
{
	State state = cml::lerp(previous, current, alpha);

	glColor3f(1.0f, 1.0f, 1.0f);
	tilemap_.bind();

	Mf::Tilemap::Index frame = animation_.getFrame();

	Mf::Scalar coords[8];
	tilemap_.getTileCoords(frame, coords);

	Mf::Scalar s = 16.0;

	glBegin(GL_TRIANGLE_FAN);
		glTexCoord2f(coords[0], coords[1]);
		glVertex3(state.position[0]-s, state.position[1]-s, z);
		glTexCoord2f(coords[2], coords[3]);
		glVertex3(state.position[0]+s, state.position[1]-s, z);
		glTexCoord2f(coords[4], coords[5]);
		glVertex3(state.position[0]+s, state.position[1]+s, z);
		glTexCoord2f(coords[6], coords[7]);
		glVertex3(state.position[0]-s, state.position[1]+s, z);
	glEnd();

	glColor3f(0.0f, 0.0f, 0.0f);
	Mf::Texture::resetBind();

	glBegin(GL_TRIANGLES);
		glVertex3(480.0, 190.0, 64.0);
		glVertex3(520.0, 190.0, 64.0);
		glVertex3(500.0, 210.0, 64.0);
	glEnd();

	glColor3f(1.0f, 1.0f, 1.0f);
}


Mf::Tilemap& Character::getTilemap()
{
	return tilemap_;
}

Mf::Animation& Character::getAnimation()
{
	return animation_;
}


/** vim: set ts=4 sw=4 tw=80: *************************************************/

